//MultiTask.sqf
//Core script of the MutliTask multiplayer-compatible task system
//Version 0.1.4
//By Deadfast (http://deadfast.armaholic.eu)
//Work licensed under Creative Commons Attribution Non-Commercial Share Alike (http://creativecommons.org/licenses/by-nc-sa/3.0/)

DFS_missionCore = _this;

waitUntil{!isNil "bis_fnc_init"};

if (isServer) then
{
	DFS_missionCore setVariable ["DFS_taskArray", [], true];
};

DFS_errorDisplay =
{
	private ["_function", "_msg"];
	_function = _this select 0;
	_msg = _this select 1;
	
	hint parseText ('<t font="Zeppelin33" align="center" size="1.5" color="#ff0000">ERROR!</t><br/>' + _function + ': ' + _msg);
	diag_log text _msg;
};
DFS_taskCommitLocal =
{
	private ["_taskName", "_task"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task = DFS_missionCore getVariable ("DFS_task_" + _taskName);
		if ( (player in (_task select 7)) || ((group player) in (_task select 8)) || ((side player) in (_task select 9)) || (count (_task select 7) == 0 && count (_task select 8) == 0 && count (_task select 9) == 0) ) then
		{
			if (isNil _taskName) then
			{ //Create if does not exist yet
				if (isNil {_task select 1}) then
				{
					call compile format ['%1 = player createSimpleTask [''%2'']', _taskName, call compile (_task select 0)];
				}
				else
				{
					call compile format ['%1 = player createSimpleTask [''%2'', %3]', _taskName, call compile (_task select 0), _task select 1];
				};
			};
			
			call compile format ['
				%1 setSimpleTaskDescription [''%2'', ''%3'', ''%4'']
			', _taskName, call compile (_task select 2), call compile (_task select 3), call compile (_task select 4)];
			
			if (!isNil {_task select 5}) then
			{
				call compile format ['%1 setSimpleTaskDestination %2', _taskName, _task select 5];
			};
			if (!isNil {_task select 6}) then
			{
				call compile format ['%1 setTaskState "%2"', _taskName, _task select 6];
			};
		}
		else
		{
			if (!isNil _taskName) then
			{
				_taskName call DFS_removeSimpleTaskLocal;
			};
		};
	}
	else
	{
		["DFS_taskCommitLocal", format ['Task variable "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
DFS_taskCommit =
{
	private "_taskName";
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		[nil,nil,rCallVar,_taskName,"DFS_taskCommitLocal"] call RE;
	}
	else
	{
		["DFS_taskCommit", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
DFS_tasksSync =
{
	waitUntil {!isNil {DFS_missionCore getVariable "DFS_taskArray"}};
	_taskArray = DFS_missionCore getVariable "DFS_taskArray";
	
	{
		_x call DFS_taskCommitLocal;
	} forEach _taskArray;
};
DFS_taskExists =
{
	private "_taskName";
	_taskName = _this;
	
	(_taskName in (DFS_missionCore getVariable "DFS_taskArray"))
};
DFS_createSimpleTask =
{
	private ["_taskName", "_title", "_parent", "_taskArray"];
	_taskName = _this select 0;
	_title = _this select 1;
	
	if (( ((toArray (toLower _taskName)) select 0 ) in ( toArray "abcdefghijklmnopqrstuvwxyz" )) ) then
	{
		_taskArray = DFS_missionCore getVariable "DFS_taskArray";
		if (! (_taskName in _taskArray) ) then
		{
			_taskArray set [count _taskArray, _taskName];
			DFS_missionCore setVariable ["DFS_taskArray", _taskArray, true];
			if (isNil {_this select 2}) then
			{
				DFS_missionCore setVariable ["DFS_task_" + _taskName, [_title, nil, nil, nil, nil, nil, "created", [], [], []], true];
			}
			else
			{
				_parent = _this select 2;
				if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _parent)}) then
				{
					DFS_missionCore setVariable ["DFS_task_" + _taskName, [_title, _parent, nil, nil, nil, nil, "created", [], [], []], true];
				}
				else
				{
					["DFS_createSimpleTask", format ['Duplicate task name "%1"', _taskName]] call DFS_errorDisplay;
				};
			};
		}
		else
		{
			["DFS_createSimpleTask", format ['Duplicate task name "%1"', _taskName]] call DFS_errorDisplay;
		};
	}
	else
	{
		["DFS_createSimpleTask", format ['Invalid task name "%1"', _taskName]] call DFS_errorDisplay;
	};
};
DFS_createTask =
{
	_this call DFS_createSimpleTask;
};
DFS_setSimpleTaskDescription =
{
	private ["_taskName", "_desc", "_title", "_caption", "_task"];
	_taskName = _this select 0;
	_desc = _this select 1;
	_title = _this select 2;
	_caption = _this select 3;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_task set [2, _desc];
		_task set [3, _title];
		_task set [4, _caption];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_setSimpleTaskDescription", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_setTaskDescription =
{
	_this call DFS_setSimpleTaskDescription;
};
DFS_taskDescription =
{
	private ["_taskName", "_task"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ DFS_missionCore getVariable ("DFS_task_" + _taskName);
		
		[_task select 2, _task select 3, _task select 4]
	}
	else
	{
		["DFS_taskDescription", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_getSimpleTaskDescription =
{
	(_this call DFS_taskDescription)
};
DFS_getTaskDescription =
{
	(_this call DFS_taskDescription)
};
DFS_setSimpleTaskDestination =
{
	private ["_taskName", "_pos", "_task"];
	_taskName = _this select 0;
	_pos = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_task set [5, _pos];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_setSimpleTaskDestination", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
DFS_setTaskDestination =
{
	_this call DFS_setSimpleTaskDestination;
};
DFS_taskDestination =
{
	private "_taskName";
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 5)
	}
	else
	{
		["DFS_taskDestination", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_getSimpleTaskDestination =
{
	(_this call DFS_taskDestination)
};
DFS_getTaskDestination =
{
	(_this call DFS_taskDestination)
};
DFS_setTaskState =
{
	private ["_taskName", "_state", "_task"];
	_taskName = _this select 0;
	_state = _this select 1;
	
	if (toLower _state in ["succeeded","failed","canceled","created","assigned"]) then
	{
		if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
		{
			_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
			_task set [6, _state];
			DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
		}
		else
		{
			["DFS_setTaskState", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
		};
	}
	else
	{
		["DFS_setTaskState", format ['Invalid task state "%1"', _state]] call DFS_errorDisplay;
	};
};
DFS_taskState =
{
	private "_taskName";
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		(toLower ((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 6))
	}
	else
	{
		["DFS_taskState", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_getTaskState =
{
	(_this call DFS_taskState)
};
DFS_setTaskUnits =
{
	private ["_taskName", "_units", "_task"];
	_taskName = _this select 0;
	_units = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_task set [7, _units];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_setTaskUnits", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_addTaskUnit =
{
	private ["_taskName", "_unit", "_task", "_units"];
	_taskName = _this select 0;
	_unit = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_units =+ (_task select 7);
		_units set [count _units, _unit];
		_task set [7, _units];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_addTaskUnit", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_taskUnits =
{
	private ["_taskName", "_task"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 7)
	}
	else
	{
		["DFS_taskUnits", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
		[]
	}
};
DFS_getTaskUnits =
{
	(_this call DFS_taskUnits)
};
DFS_setTaskGroups =
{
	private ["_taskName", "_groups", "_task"];
	_taskName = _this select 0;
	_groups = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_task set [8, _groups];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_setTaskGroups", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_addTaskGroup =
{
	private ["_taskName", "_group", "_task", "_groups"];
	_taskName = _this select 0;
	_group = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_groups =+ (_task select 8);
		_groups set [count _groups, _group];
		_task set [8, _groups];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_addTaskGroup", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_taskGroups =
{
	private ["_taskName", "_task"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 8)
	}
	else
	{
		["DFS_taskGroups", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
		[]
	}
};
DFS_getTaskGroups =
{
	(_this call DFS_taskGroups)
};
DFS_setTaskSides =
{
	private ["_taskName", "_sides", "_task"];
	_taskName = _this select 0;
	_sides = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_task set [9, _sides];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_setTaskSides", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_addTaskSide =
{
	private ["_taskName", "_side", "_task", "_sides"];
	_taskName = _this select 0;
	_side = _this select 1;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_task =+ (DFS_missionCore getVariable ("DFS_task_" + _taskName));
		_sides =+ (_task select 9);
		_sides set [count _sides, _side];
		_task set [9, _sides];
		DFS_missionCore setVariable ["DFS_task_" + _taskName, _task, true];
	}
	else
	{
		["DFS_addTaskSide", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	}
};
DFS_taskSides =
{
	private ["_taskName", "_task"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 9)
	}
	else
	{
		["DFS_taskSides", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
		[]
	}
};
DFS_getTaskSides =
{
	(_this call DFS_taskSides)
};
DFS_removeSimpleTaskLocal =
{
	private ["_taskName", "_msg"];
	_taskName = _this;
	
	if (!isNil _taskName) then
	{
		call compile format ['
			player removeSimpleTask %1;
			%1 = nil;
		', _taskName];
	}
	else
	{
		[format ['DFS_removeSimpleTaskLocal: Task variable "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
DFS_removeTaskLocal =
{
	_this call DFS_removeSimpleTaskLocal;
};
DFS_removeSimpleTask =
{
	private ["_taskName", "_taskArray"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		_taskArray =+ (DFS_missionCore getVariable "DFS_taskArray");
		_taskArray = _taskArray - [_taskName];
		DFS_missionCore setVariable ["DFS_taskArray", _taskArray, true];
		
		DFS_missionCore setVariable ["DFS_task_" + _taskName, nil];
		[nil,nil,rCallVar,_taskName,"DFS_removeSimpleTaskLocal"] call RE;
	}
	else
	{
		["DFS_removeSimpleTask", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
DFS_removeTask =
{
	_this call DFS_removeSimpleTask;
};
DFS_taskHintLocal =
{
	#define __WHITE [1,1,1,1]
	#define __GREY [0.75,0.75,0.75,1]
	#define __GREEN [0.6,0.8,0.4,1]
	#define __RED [1,0.1,0,1]
	if (!isDedicated) then
	{
		private ["_taskName", "_params"];
		_taskName = _this;
		
		if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
		{
			if (!isNil _taskName) then
			{
				_params = switch (_taskName call DFS_taskState) do
				{
					case "created":
					{
						["str_taskNew", __WHITE, "taskNew"]
					};
					case "current":
					{
						["str_taskSetCurrent", __WHITE, "taskCurrent"]
					};
					case "assigned":
					{
						["str_taskSetCurrent", __WHITE, "taskCurrent"]
					};
					case "succeeded":
					{
						["str_taskAccomplished", __GREEN, "taskDone"]
					};
					case "failed":
					{
						["str_taskFailed", __RED, "taskFailed"]
					};
					case "canceled":
					{
						["str_taskCancelled", __GREY, "taskDone"]
					};
				};
				
				taskHint [(localize (_params select 0)) + "\n" + (call compile ((DFS_missionCore getVariable ("DFS_task_" + _taskName)) select 3)), _params select 1, _params select 2];
			};
		}
		else
		{
			["DFS_taskHintLocal", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
		};
	};
};
DFS_taskHint =
{
	private ["_taskName", "_msg"];
	_taskName = _this;
	
	if (!isNil {DFS_missionCore getVariable ("DFS_task_" + _taskName)}) then
	{
		[nil,nil,rCallVar,_taskName,"DFS_taskHintLocal"] call RE;
	}
	else
	{
		["DFS_taskHint", format ['Task "%1" does not exist', _taskName]] call DFS_errorDisplay;
	};
};
